@@ -1,44 +1,11 @@
 import numpy as np
 # from numba import jit
 import warnings
-
-
-# @jit(nopython=True, cache=True)
-# def __signal_orthogonalization_numba(signal_1, signal_2):
-#     """Piece of code to calculate the signal orthogonalization matrix using
-#     numba
-#     """
-#     # Create C contiguous numpy arrays to increase performance
-#     signal_1 = signal_1.copy()
-#     signal_2 = signal_2.copy()
-#
-#     # Init
-#     n_samp = signal_1.shape[0]
-#     n_cha_s1 = signal_1.shape[1]
-#     n_cha_s2 = signal_1.shape[1]
-#
-#     signal_ort = np.empty((n_samp, n_cha_s2, n_cha_s1))
-#     signal_ort[:] = np.nan
-#
-#     # Orthogonalization
-#     for cha1 in range(n_cha_s1):
-#         for cha2 in range(n_cha_s2):
-#             signal_1_cha = signal_1[:, cha1]
-#             signal_2_cha = signal_2[:, cha2]
-#             if not np.array_equal(signal_1_cha, signal_2_cha):
-#                 signal_1_cha_inv = np.linalg.pinv(signal_1_cha.reshape(-1, 1))
-#                 signal_1_cha_inv = signal_1_cha_inv.reshape(
-#                     signal_1_cha_inv.shape[1]
-#                 )
-#                 beta = np.dot(signal_2_cha, signal_1_cha_inv)
-#                 signal_ort[:, cha2, cha1] = \
-#                     signal_2_cha - beta * signal_1_cha
-#             else:
-#                 signal_ort[:, cha2, cha1] = signal_1_cha
-#     return signal_ort
+from medusa.utils import check_dimensions
 
 
 def signal_orthogonalization_cpu(signal_1, signal_2):
+
     """ This method implements the ortogonalization of each channel of signal_1
     regarding all the channels in signal_2 using CPU
 
@@ -50,18 +17,21 @@ def signal_orthogonalization_cpu(signal_1, signal_2):
     Parameters
     ----------
     signal_1 : numpy 2D matrix
-        First MEEG Signal. [n_samples x n_channels].
+        First MEEG Signal. Allowed dimensions: [n_epochs, n_samples, n_channels],
+        [n_samples, n_channels] and [n_samples].
     signal_2 : numpy 2D matrix
         Second MEEG Signal. If empty signal_2 will be set to be equal to
-        signal_1. [n_samples x n_channels].
+        signal_1. Allowed dimensions: [n_epochs, n_samples, n_channels],
+        [n_samples, n_channels] and [n_samples].
 
     Returns
     -------
     signal_ort : numpy 3D matrix
-        MEEG ortogonalised signals. The first dimension is samples. The third
-        dimension is the channel of first signal, and the second dimension is
-        regarding which channel of second signal it has been orthogonalized the
-        channel in third dimension. [n_samples x n_channels x n_channels].
+        MEEG ortogonalised signals. The first dimension is epochs. The second
+        dimension is samples. The fourth dimension is the channel of first signal,
+        and the second dimension is regarding which channel of second signal it
+        has been orthogonalized the channel in third dimension.
+        [n_epochs, n_samples, n_channels, n_channels].
 
     """
     # Error check
@@ -72,43 +42,111 @@ def signal_orthogonalization_cpu(signal_1, signal_2):
     if (type(signal_1) != np.ndarray) or (type(signal_2) != np.ndarray):
         raise ValueError("Parameter signal_1 and signal_2 must be of type "
                          "numpy.ndarray")
-    if signal_1.shape[0] < signal_1.shape[1]:
-        warnings.warn("Warning: Signal dimensions flipped out. If you have more"
-                      " samples than channels, comment this "
-                      "line")
-        signal_1 = signal_1.T
-    if signal_2.shape[0] < signal_2.shape[1]:
-        warnings.warn(
-            "Warning: Signal dimensions flipped out. If you have more samples"
-            " than channels, comment this "
-            "line")
-        signal_2 = signal_2.T
 
-    # Variable initialization
-    signal_ort = np.empty((signal_1.shape[0], signal_2.shape[1],
-                           signal_1.shape[1]))
-    signal_ort[:] = np.nan
+    # Set correct dimensions
+    signal_1 = np.ascontiguousarray(check_dimensions(signal_1))
+    signal_2 = np.ascontiguousarray(check_dimensions(signal_2))
 
-    # Orthogonalization process
-    for chan_1 in range(0, signal_1.shape[1]):
-        for chan_2 in range(0, signal_2.shape[1]):
-            signal_1_chan = signal_1[:, chan_1]
-            signal_2_chan = signal_2[:, chan_2]
-            if not np.array_equal(signal_1_chan, signal_2_chan):
-                beta = np.matmul(signal_2_chan[None],
-                                 np.linalg.pinv(signal_1_chan[None]))
-                signal_ort[:, chan_2, chan_1] = signal_2_chan - beta * \
-                                                signal_1_chan
-            else:
-                signal_ort[:, chan_2, chan_1] = signal_1_chan
+    n_chan_1 = signal_1.shape[2]
+    n_chan_2 = signal_2.shape[2]
+    n_samp_1 = signal_1.shape[1]
+    n_epo_1 = signal_1.shape[0]
 
-    # signal_ort = __signal_orthogonalization_numba(signal_1, signal_2)
+    signal_1_chan = np.transpose(np.repeat(signal_1, repeats=n_chan_1, axis=2), (0,2,1))
+    signal_2_chan = np.transpose(np.tile(signal_2, (1, 1, n_chan_2)),(0,2,1))
+
+    # Add extra dimensions
+    signal_1_chan = signal_1_chan[:,:,np.newaxis,:]
+    signal_2_chan = signal_2_chan[:,:,np.newaxis,:]
+
+    # Calculate pseudo inverse
+    inv_signal_1_chan = np.repeat(np.linalg.pinv(np.transpose(signal_1,(0,2,1))[:, :,
+                                       np.newaxis, :]),repeats=n_chan_1,axis=1)
+    beta = np.matmul(signal_2_chan, inv_signal_1_chan)
+
+    signal_ort = np.transpose(np.reshape(signal_2_chan - beta * signal_1_chan,
+                            (n_epo_1,n_chan_2,n_chan_1,n_samp_1),order='F'),(0,3,1,2))
+
+    signal_ort[:,:,np.arange(n_chan_2),np.arange(n_chan_1)] = signal_1
+
+    return signal_ort
+
+def signal_orthogonalization_gpu(signal_1, signal_2):
+    """
+    This function ortogonalizes each channel of signal_1 regarding all the
+    channels in signal_2. Based in O'Neill et al. 2015
+
+    Parameters
+    ----------
+    signal_1 : numpy 3D matrix
+        First MEEG Signal. Allowed dimensions: [n_epochs, n_samples, n_channels],
+        [n_samples, n_channels] and [n_samples].
+    signal_2 : numpy 3D matrix
+        Second MEEG Signal. If empty signal_2 will be set to be equal to
+        signal_1. Allowed dimensions: [n_epochs, n_samples, n_channels],
+        [n_samples, n_channels] and [n_samples].
+
+    Returns
+    -------
+    signal_ort : numpy 3D matrix
+        MEEG ortogonalized signals. The first dimension is epochs. The second
+        dimension is samples. The fourth dimension is the channel of first signal,
+        and the second dimension is regarding which channel of second signal it
+        has been orthogonalized the channel in third dimension.
+        [n_epochs, n_samples, n_channels, n_channels].
+    """
+    import tensorflow as tf
+
+    # Error check
+    if signal_2 is None:
+        signal_2 = signal_1
+    if len(signal_1) != len(signal_2):
+        raise ValueError("Signals must have the same length")
+    if (type(signal_1) != np.ndarray) or (type(signal_2) != np.ndarray):
+        raise ValueError("Parameter signal_1 and signal_2 must be of type "
+                         "numpy.ndarray")
+
+    # Set correct dimensions
+    signal_1 = np.ascontiguousarray(check_dimensions(signal_1))
+    signal_2 = np.ascontiguousarray(check_dimensions(signal_2))
+
+    n_chan_1 = signal_1.shape[2]
+    n_chan_2 = signal_2.shape[2]
+    n_samp_1 = signal_1.shape[1]
+    n_epo_1 = signal_1.shape[0]
+
+    signal_1_chan = tf.transpose(tf.repeat(signal_1, repeats=n_chan_1, axis=2),
+                                 perm=[0, 2, 1])
+    signal_2_chan = tf.transpose(tf.tile(signal_2,(1,1,n_chan_2)),perm=[0,2,1])
+
+    # Add extra dimensions
+    signal_1_chan = signal_1_chan[:,:,tf.newaxis,:]
+    signal_2_chan = signal_2_chan[:,:,tf.newaxis,:]
+
+    # Calculate pseudo inverse
+    inv_signal_1_chan = tf.repeat(
+        tf.linalg.pinv(tf.transpose(signal_1, perm=[0, 2, 1])[:, :,
+                       np.newaxis, :]), repeats=n_chan_1, axis=1)
+    beta = np.matmul(signal_2_chan, inv_signal_1_chan)
+
+    signal_ort = tf.transpose(tf.transpose(tf.reshape(tf.transpose(signal_2_chan - beta *
+                                         signal_1_chan,perm=[0,2,3,1]),(n_epo_1,
+                                                                        n_samp_1,
+                                                                        n_chan_2,
+                                                                        n_chan_1)),
+                              perm =(0,3,2,1)),perm=[0,3,1,2]).numpy()
+
+    signal_ort[:,:,tf.range(n_chan_2),tf.range(n_chan_1)] = signal_1
 
     return signal_ort
 
 
-def signal_orthogonalization_gpu(signal_1, signal_2):
+def signal_orthogonalization_gpu_old(signal_1, signal_2):
     """
+    DEPRECATED: This functions performs the orthogonalization of the signal
+    but it is not vectorized nor allow epochs as inputs. Its use is not
+    recommended.
+
     This function ortogonalizes each channel of signal_1 regarding all the
     channels in signal_2. Based in O'Neill et al. 2015
 
@@ -131,6 +169,8 @@ def signal_orthogonalization_gpu(signal_1, signal_2):
 
     import tensorflow as tf
     # Error check
+    if signal_2 is None:
+        signal_2 = signal_1
     if len(signal_1) != len(signal_2):
         raise ValueError("Signals must have the same length")
     if (type(signal_1) != np.ndarray) or (type(signal_2) != np.ndarray):
